# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

import flatbuffers

class EvaluatedFunction(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsEvaluatedFunction(cls, buf, offset):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = EvaluatedFunction()
        x.Init(buf, n + offset)
        return x

    # EvaluatedFunction
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # EvaluatedFunction
    def ValuesType(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, o + self._tab.Pos)
        return 0

    # EvaluatedFunction
    def Values(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            from flatbuffers.table import Table
            obj = Table(bytearray(), 0)
            self._tab.Union(obj, o)
            return obj
        return None

    # EvaluatedFunction
    def DerivativesType(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, o + self._tab.Pos)
        return 0

    # EvaluatedFunction
    def Derivatives(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            from flatbuffers.table import Table
            obj = Table(bytearray(), 0)
            self._tab.Union(obj, o)
            return obj
        return None

    # EvaluatedFunction
    def Errors(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .Quantiles import Quantiles
            obj = Quantiles()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # EvaluatedFunction
    def ErrorsLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

def EvaluatedFunctionStart(builder): builder.StartObject(5)
def EvaluatedFunctionAddValuesType(builder, valuesType): builder.PrependUint8Slot(0, valuesType, 0)
def EvaluatedFunctionAddValues(builder, values): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(values), 0)
def EvaluatedFunctionAddDerivativesType(builder, derivativesType): builder.PrependUint8Slot(2, derivativesType, 0)
def EvaluatedFunctionAddDerivatives(builder, derivatives): builder.PrependUOffsetTRelativeSlot(3, flatbuffers.number_types.UOffsetTFlags.py_type(derivatives), 0)
def EvaluatedFunctionAddErrors(builder, errors): builder.PrependUOffsetTRelativeSlot(4, flatbuffers.number_types.UOffsetTFlags.py_type(errors), 0)
def EvaluatedFunctionStartErrorsVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def EvaluatedFunctionEnd(builder): return builder.EndObject()



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

import flatbuffers

class Variation(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsVariation(cls, buf, offset):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = Variation()
        x.Init(buf, n + offset)
        return x

    # Variation
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Variation
    def Assignments(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .Assignment import Assignment
            obj = Assignment()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Variation
    def AssignmentsLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # Variation
    def Systematic(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Float64Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 8))
        return 0

    # Variation
    def SystematicAsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Float64Flags, o)
        return 0

    # Variation
    def SystematicLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # Variation
    def CategorySystematic(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.String(a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 4))
        return ""

    # Variation
    def CategorySystematicLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

def VariationStart(builder): builder.StartObject(3)
def VariationAddAssignments(builder, assignments): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(assignments), 0)
def VariationStartAssignmentsVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def VariationAddSystematic(builder, systematic): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(systematic), 0)
def VariationStartSystematicVector(builder, numElems): return builder.StartVector(8, numElems, 8)
def VariationAddCategorySystematic(builder, categorySystematic): builder.PrependUOffsetTRelativeSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(categorySystematic), 0)
def VariationStartCategorySystematicVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def VariationEnd(builder): return builder.EndObject()



#!/usr/bin/env python

# BSD 3-Clause License; see https://github.com/scikit-hep/aghast/blob/master/LICENSE

from aghast.interface import frombuffer
from aghast.interface import fromarray
from aghast.interface import fromfile

def to_numpy(obj):
    import aghast._connect._numpy
    return aghast._connect._numpy.to_numpy(obj)

def from_numpy(obj):
    import aghast._connect._numpy
    return aghast._connect._numpy.from_numpy(obj)

def to_pandas(obj):
    import aghast._connect._pandas
    return aghast._connect._pandas.to_pandas(obj)

def from_pandas(obj):
    import aghast._connect._pandas
    return aghast._connect._pandas.from_pandas(obj)

def to_root(obj, name):
    import aghast._connect._root
    return aghast._connect._root.to_root(obj, name)

def from_root(obj, collection=False):
    import aghast._connect._root
    return aghast._connect._root.from_root(obj, collection=collection)

def to_fnalhist(obj):
    import aghast._connect._fnalhist
    return aghast._connect._fnalhist.to_fnalhist(obj)

def from_fnalhist(obj):
    import aghast._connect._fnalhist
    return aghast._connect._fnalhist.from_fnalhist(obj)

from aghast.interface import Metadata
from aghast.interface import Decoration
from aghast.interface import Buffer
from aghast.interface import ExternalBuffer
from aghast.interface import Interpretation
from aghast.interface import InterpretedBuffer
from aghast.interface import RawInlineBuffer
from aghast.interface import RawExternalBuffer
from aghast.interface import InterpretedInlineBuffer
from aghast.interface import InterpretedInlineInt64Buffer
from aghast.interface import InterpretedInlineFloat64Buffer
from aghast.interface import InterpretedExternalBuffer
from aghast.interface import StatisticFilter
from aghast.interface import Moments
from aghast.interface import Extremes
from aghast.interface import Quantiles
from aghast.interface import Modes
from aghast.interface import Statistics
from aghast.interface import Covariance
from aghast.interface import BinLocation
from aghast.interface import IntegerBinning
from aghast.interface import RealInterval
from aghast.interface import RealOverflow
from aghast.interface import RegularBinning
from aghast.interface import HexagonalBinning
from aghast.interface import EdgesBinning
from aghast.interface import IrregularBinning
from aghast.interface import CategoryBinning
from aghast.interface import SparseRegularBinning
from aghast.interface import FractionBinning
from aghast.interface import PredicateBinning
from aghast.interface import Assignment
from aghast.interface import Variation
from aghast.interface import VariationBinning
from aghast.interface import Axis
from aghast.interface import Profile
from aghast.interface import UnweightedCounts
from aghast.interface import WeightedCounts
from aghast.interface import Parameter
from aghast.interface import ParameterizedFunction
from aghast.interface import EvaluatedFunction
from aghast.interface import BinnedEvaluatedFunction
from aghast.interface import Histogram
from aghast.interface import Page
from aghast.interface import ColumnChunk
from aghast.interface import Chunk
from aghast.interface import Column
from aghast.interface import NtupleInstance
from aghast.interface import Ntuple
from aghast.interface import Collection



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

import flatbuffers

class InterpretedExternalBuffer(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsInterpretedExternalBuffer(cls, buf, offset):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = InterpretedExternalBuffer()
        x.Init(buf, n + offset)
        return x

    # InterpretedExternalBuffer
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # InterpretedExternalBuffer
    def Pointer(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint64Flags, o + self._tab.Pos)
        return 0

    # InterpretedExternalBuffer
    def Numbytes(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint64Flags, o + self._tab.Pos)
        return 0

    # InterpretedExternalBuffer
    def ExternalSource(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint32Flags, o + self._tab.Pos)
        return 0

    # InterpretedExternalBuffer
    def Filters(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Uint32Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 4))
        return 0

    # InterpretedExternalBuffer
    def FiltersAsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Uint32Flags, o)
        return 0

    # InterpretedExternalBuffer
    def FiltersLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # InterpretedExternalBuffer
    def PostfilterSlice(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            x = o + self._tab.Pos
            from .Slice import Slice
            obj = Slice()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # InterpretedExternalBuffer
    def Dtype(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int8Flags, o + self._tab.Pos)
        return 0

    # InterpretedExternalBuffer
    def Endianness(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int8Flags, o + self._tab.Pos)
        return 0

    # InterpretedExternalBuffer
    def DimensionOrder(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(18))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int8Flags, o + self._tab.Pos)
        return 0

    # InterpretedExternalBuffer
    def Location(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(20))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

def InterpretedExternalBufferStart(builder): builder.StartObject(9)
def InterpretedExternalBufferAddPointer(builder, pointer): builder.PrependUint64Slot(0, pointer, 0)
def InterpretedExternalBufferAddNumbytes(builder, numbytes): builder.PrependUint64Slot(1, numbytes, 0)
def InterpretedExternalBufferAddExternalSource(builder, externalSource): builder.PrependUint32Slot(2, externalSource, 0)
def InterpretedExternalBufferAddFilters(builder, filters): builder.PrependUOffsetTRelativeSlot(3, flatbuffers.number_types.UOffsetTFlags.py_type(filters), 0)
def InterpretedExternalBufferStartFiltersVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def InterpretedExternalBufferAddPostfilterSlice(builder, postfilterSlice): builder.PrependStructSlot(4, flatbuffers.number_types.UOffsetTFlags.py_type(postfilterSlice), 0)
def InterpretedExternalBufferAddDtype(builder, dtype): builder.PrependInt8Slot(5, dtype, 0)
def InterpretedExternalBufferAddEndianness(builder, endianness): builder.PrependInt8Slot(6, endianness, 0)
def InterpretedExternalBufferAddDimensionOrder(builder, dimensionOrder): builder.PrependInt8Slot(7, dimensionOrder, 0)
def InterpretedExternalBufferAddLocation(builder, location): builder.PrependUOffsetTRelativeSlot(8, flatbuffers.number_types.UOffsetTFlags.py_type(location), 0)
def InterpretedExternalBufferEnd(builder): return builder.EndObject()



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

class SystematicUnits(object):
    syst_unspecified = 0
    syst_confidence = 1
    syst_sigmas = 2




# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

class ObjectData(object):
    NONE = 0
    Histogram = 1
    Ntuple = 2
    FunctionObject = 3
    Collection = 4




# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

import flatbuffers

# ///////////////////////////////////////////////// functions
class Parameter(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsParameter(cls, buf, offset):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = Parameter()
        x.Init(buf, n + offset)
        return x

    # Parameter
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Parameter
    def Identifier(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Parameter
    def ValuesType(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, o + self._tab.Pos)
        return 0

    # Parameter
    def Values(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            from flatbuffers.table import Table
            obj = Table(bytearray(), 0)
            self._tab.Union(obj, o)
            return obj
        return None

    # Parameter
    def ErrorsType(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, o + self._tab.Pos)
        return 0

    # Parameter
    def Errors(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            from flatbuffers.table import Table
            obj = Table(bytearray(), 0)
            self._tab.Union(obj, o)
            return obj
        return None

def ParameterStart(builder): builder.StartObject(5)
def ParameterAddIdentifier(builder, identifier): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(identifier), 0)
def ParameterAddValuesType(builder, valuesType): builder.PrependUint8Slot(1, valuesType, 0)
def ParameterAddValues(builder, values): builder.PrependUOffsetTRelativeSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(values), 0)
def ParameterAddErrorsType(builder, errorsType): builder.PrependUint8Slot(3, errorsType, 0)
def ParameterAddErrors(builder, errors): builder.PrependUOffsetTRelativeSlot(4, flatbuffers.number_types.UOffsetTFlags.py_type(errors), 0)
def ParameterEnd(builder): return builder.EndObject()



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

# ///////////////////////////////////////////////// metadata
class MetadataLanguage(object):
    meta_unspecified = 0
    meta_json = 1




#!/usr/bin/env python

# BSD 3-Clause License; see https://github.com/scikit-hep/aghast/blob/master/LICENSE

import unittest

import numpy

from aghast import *

class Test(unittest.TestCase):
    def runTest(self):
        pass

    def test_validity_Metadata(self):
        h = Collection({}, metadata=Metadata("""{"one": 1, "two": 2}""", language=Metadata.json))
        h.checkvalid()
        assert h.metadata.data == """{"one": 1, "two": 2}"""
        assert h.metadata.language == Metadata.json

    def test_validity_Decoration(self):
        h = Collection({}, decoration=Decoration("""points { color: red }""", language=Decoration.css))
        h.checkvalid()
        assert h.decoration.data == """points { color: red }"""
        assert h.decoration.css == Decoration.css

    def test_validity_RawInlineBuffer(self):
        h = Ntuple([Column("one", Column.int32)], [NtupleInstance([Chunk([ColumnChunk([Page(RawInlineBuffer(numpy.zeros(1, dtype=numpy.int32)))], [0, 1])])])])
        h.checkvalid()
        assert len(h.instances[0].chunks[0].column_chunks[0].pages[0].array) == 1

        h = Ntuple([Column("one", Column.int32)], [NtupleInstance([Chunk([ColumnChunk([Page(RawInlineBuffer(b"\x05\x00\x00\x00"))], [0, 1])])])])
        h.checkvalid()
        assert h.instances[0].chunks[0].column_chunks[0].pages[0].array.tolist() == [5]

    def test_validity_RawExternalBuffer(self):
        buf = numpy.zeros(1, dtype=numpy.int32)
        h = Ntuple([Column("one", Column.int32)], [NtupleInstance([Chunk([ColumnChunk([Page(RawExternalBuffer(buf.ctypes.data, buf.nbytes))], [0, 1])])])])
        h.checkvalid()
        assert len(h.instances[0].chunks[0].column_chunks[0].pages[0].array) == 1

        buf = numpy.array([3.14], dtype=numpy.float64)
        h = Ntuple([Column("one", Column.float64)], [NtupleInstance([Chunk([ColumnChunk([Page(RawExternalBuffer(buf.ctypes.data, buf.nbytes))], [0, 1])])])])
        h.checkvalid()
        assert h.instances[0].chunks[0].column_chunks[0].pages[0].array.tolist() == [3.14]

    def test_validity_InterpretedInlineBuffer(self):
        h = BinnedEvaluatedFunction([Axis()], InterpretedInlineBuffer(numpy.zeros(1, dtype=numpy.int32), dtype=InterpretedInlineBuffer.int32))
        h.checkvalid()
        assert h.values.array.tolist() == [0]

        h = BinnedEvaluatedFunction([Axis()], InterpretedInlineBuffer(b"\x07\x00\x00\x00", dtype=InterpretedInlineBuffer.int32))
        h.checkvalid()
        assert h.values.array.tolist() == [7]

    def test_validity_InterpretedExternalBuffer(self):
        buf = numpy.zeros(1, dtype=numpy.float64)
        h = BinnedEvaluatedFunction([Axis()], InterpretedExternalBuffer(buf.ctypes.data, buf.nbytes, dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [0.0]

        buf = numpy.array([3.14], dtype=numpy.float64)
        h = BinnedEvaluatedFunction([Axis()], InterpretedExternalBuffer(buf.ctypes.data, buf.nbytes, dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [3.14]

    def test_validity_IntegerBinning(self):
        h = BinnedEvaluatedFunction([Axis(IntegerBinning(10, 20))], InterpretedInlineBuffer(numpy.zeros(11), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        h = BinnedEvaluatedFunction([Axis(IntegerBinning(20, 10))], InterpretedInlineBuffer(numpy.zeros(11), dtype=InterpretedInlineBuffer.float64))
        assert not h.isvalid
        h = BinnedEvaluatedFunction([Axis(IntegerBinning(10, 20, loc_underflow=IntegerBinning.nonexistent, loc_overflow=IntegerBinning.nonexistent))], InterpretedInlineBuffer(numpy.zeros(11), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [0.0] * 11
        h = BinnedEvaluatedFunction([Axis(IntegerBinning(10, 20, loc_underflow=IntegerBinning.below1, loc_overflow=IntegerBinning.nonexistent))], InterpretedInlineBuffer(numpy.zeros(12), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [0.0] * 12
        h = BinnedEvaluatedFunction([Axis(IntegerBinning(10, 20, loc_underflow=IntegerBinning.nonexistent, loc_overflow=IntegerBinning.above1))], InterpretedInlineBuffer(numpy.zeros(12), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [0.0] * 12
        h = BinnedEvaluatedFunction([Axis(IntegerBinning(10, 20, loc_underflow=IntegerBinning.below1, loc_overflow=IntegerBinning.above1))], InterpretedInlineBuffer(numpy.zeros(13), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [0.0] * 13

    def test_validity_RealInterval(self):
        h = BinnedEvaluatedFunction([Axis(RegularBinning(10, RealInterval(-5, 5)))], InterpretedInlineBuffer(numpy.zeros(10), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        h = BinnedEvaluatedFunction([Axis(RegularBinning(10, RealInterval(5, -5)))], InterpretedInlineBuffer(numpy.zeros(10), dtype=InterpretedInlineBuffer.float64))
        assert not h.isvalid

    def test_validity_RealOverflow(self):
        h = BinnedEvaluatedFunction([Axis(RegularBinning(10, RealInterval(-5, 5), RealOverflow(loc_underflow=RealOverflow.nonexistent, loc_overflow=RealOverflow.nonexistent, loc_nanflow=RealOverflow.nonexistent)))], InterpretedInlineBuffer(numpy.zeros(10), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [0.0] * 10
        h = BinnedEvaluatedFunction([Axis(RegularBinning(10, RealInterval(-5, 5), RealOverflow(loc_underflow=RealOverflow.above1, loc_overflow=RealOverflow.nonexistent, loc_nanflow=RealOverflow.nonexistent)))], InterpretedInlineBuffer(numpy.zeros(11), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [0.0] * 11
        h = BinnedEvaluatedFunction([Axis(RegularBinning(10, RealInterval(-5, 5), RealOverflow(loc_underflow=RealOverflow.nonexistent, loc_overflow=RealOverflow.above1, loc_nanflow=RealOverflow.nonexistent)))], InterpretedInlineBuffer(numpy.zeros(11), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [0.0] * 11
        h = BinnedEvaluatedFunction([Axis(RegularBinning(10, RealInterval(-5, 5), RealOverflow(loc_underflow=RealOverflow.nonexistent, loc_overflow=RealOverflow.nonexistent, loc_nanflow=RealOverflow.above1)))], InterpretedInlineBuffer(numpy.zeros(11), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [0.0] * 11
        h = BinnedEvaluatedFunction([Axis(RegularBinning(10, RealInterval(-5, 5), RealOverflow(loc_underflow=RealOverflow.above1, loc_overflow=RealOverflow.nonexistent, loc_nanflow=RealOverflow.above2)))], InterpretedInlineBuffer(numpy.zeros(12), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [0.0] * 12
        h = BinnedEvaluatedFunction([Axis(RegularBinning(10, RealInterval(-5, 5), RealOverflow(loc_underflow=RealOverflow.above1, loc_overflow=RealOverflow.above2, loc_nanflow=RealOverflow.above3)))], InterpretedInlineBuffer(numpy.zeros(13), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [0.0] * 13

    def test_validity_RegularBinning(self):
        h = BinnedEvaluatedFunction([Axis(RegularBinning(10, RealInterval(-5, 5)))], InterpretedInlineBuffer(numpy.zeros(10), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()

    def test_validity_HexagonalBinning(self):
        h = BinnedEvaluatedFunction([Axis(HexagonalBinning(3, 5, -5, -4))], InterpretedInlineBuffer(numpy.array([[0.0] * 2] * 3), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [[0.0] * 2] * 3
        h = BinnedEvaluatedFunction([Axis(HexagonalBinning(3, 5, -5, -4, qoverflow=RealOverflow(loc_nanflow=RealOverflow.above1)))], InterpretedInlineBuffer(numpy.array([[0.0] * 2] * 4), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [[0.0] * 2] * 4
        h = BinnedEvaluatedFunction([Axis(HexagonalBinning(3, 5, -5, -4, roverflow=RealOverflow(loc_nanflow=RealOverflow.above1)))], InterpretedInlineBuffer(numpy.array([[0.0] * 3] * 3), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [[0.0] * 3] * 3
        h = BinnedEvaluatedFunction([Axis(HexagonalBinning(3, 5, -5, -4, qoverflow=RealOverflow(loc_nanflow=RealOverflow.above1), roverflow=RealOverflow(loc_nanflow=RealOverflow.above1)))], InterpretedInlineBuffer(numpy.array([[0.0] * 3] * 4), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [[0.0] * 3] * 4

    def test_validity_EdgesBinning(self):
        h = BinnedEvaluatedFunction([Axis(EdgesBinning([3.3], overflow=RealOverflow(loc_underflow=RealOverflow.above1, loc_overflow=RealOverflow.above2)))], InterpretedInlineBuffer(numpy.array([0.0, 0.0]), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [0.0, 0.0]
        h = BinnedEvaluatedFunction([Axis(EdgesBinning([1.1, 2.2, 3.3]))], InterpretedInlineBuffer(numpy.array([0.0, 0.0]), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [0.0, 0.0]

    def test_validity_IrregularBinning(self):
        h = BinnedEvaluatedFunction([Axis(IrregularBinning([RealInterval(0.5, 1.5)]))], InterpretedInlineBuffer(numpy.array([0.0]), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [0.0]
        h = BinnedEvaluatedFunction([Axis(IrregularBinning([RealInterval(0.5, 1.5), RealInterval(1.5, 1.5), RealInterval(0.0, 10.0)]))], InterpretedInlineBuffer(numpy.array([0.0, 0.0, 0.0]), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [0.0, 0.0, 0.0]

    def test_validity_CategoryBinning(self):
        h = BinnedEvaluatedFunction([Axis(CategoryBinning(["one", "two", "three"]))], InterpretedInlineBuffer(numpy.array([0.0, 0.0, 0.0]), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [0.0, 0.0, 0.0]
        h = BinnedEvaluatedFunction([Axis(CategoryBinning(["one", "two", "three"], loc_overflow=CategoryBinning.above1))], InterpretedInlineBuffer(numpy.array([0.0, 0.0, 0.0, 0.0]), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [0.0, 0.0, 0.0, 0.0]

    def test_validity_SparseRegularBinning(self):
        h = BinnedEvaluatedFunction([Axis(SparseRegularBinning([-5, -3, 10, 1000], 0.1))], InterpretedInlineBuffer(numpy.array([0.0, 0.0, 0.0, 0.0]), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [0.0, 0.0, 0.0, 0.0]

    def test_validity_FractionBinning(self):
        h = BinnedEvaluatedFunction([Axis(FractionBinning())], InterpretedInlineBuffer(numpy.array([0.0, 0.0]), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [0.0, 0.0]
        assert h.axis[0].binning.error_method == FractionBinning.unspecified
        h = BinnedEvaluatedFunction([Axis(FractionBinning()), Axis(RegularBinning(10, RealInterval(-5, 5)))], InterpretedInlineBuffer(numpy.array([[0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]]), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [[0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]]
        h = BinnedEvaluatedFunction([Axis(RegularBinning(10, RealInterval(-5, 5))), Axis(FractionBinning())], InterpretedInlineBuffer(numpy.array([[[0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]]]), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [[0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0]]

    def test_validity_PredicateBinning(self):
        h = Histogram([Axis(PredicateBinning(["p", "q"]))], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.array([0.0, 0.0]))))
        h.checkvalid()

    def test_validity_Assignments(self):
        h = Histogram([Axis(VariationBinning([Variation([Assignment("x", "1"), Assignment("y", "2"), Assignment("z", "3")])]))], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.array([0.0]))))
        h.checkvalid()
        assert h.axis[0].binning.variations[0].assignments[1].expression == "2"

    def test_validity_Variation(self):
        h = Histogram([Axis(VariationBinning([Variation([Assignment("x", "1")]), Variation([Assignment("x", "2")])]))], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.array([0.0, 0.0]))))
        h.checkvalid()

    def test_validity_VariationBinning(self):
        h = Histogram([Axis(VariationBinning([Variation([Assignment("x", "1")]), Variation([Assignment("x", "2")]), Variation([Assignment("x", "3")])]))], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.array([0.0, 0.0, 0.0]))))
        h.checkvalid()

    def test_validity_Axis(self):
        h = BinnedEvaluatedFunction([Axis(expression="x", title="wow")], InterpretedInlineBuffer(numpy.array([0.0]), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.axis[0].expression == "x"
        assert h.values.array.tolist() == [0.0]

    def test_validity_UnweightedCounts(self):
        h = Histogram([Axis(RegularBinning(10, RealInterval(-5, 5)))], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(10))))
        h.checkvalid()

    def test_validity_WeightedCounts(self):
        h = Histogram([Axis(RegularBinning(10, RealInterval(-5, 5)))], WeightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(10))))
        h.checkvalid()
        h = Histogram([Axis(RegularBinning(10, RealInterval(-5, 5)))], WeightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(10)), sumw2=InterpretedInlineBuffer.fromarray(numpy.arange(10)**2)))
        h.checkvalid()
        h = Histogram([Axis(RegularBinning(10, RealInterval(-5, 5)))], WeightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(10)), sumw2=InterpretedInlineBuffer.fromarray(numpy.arange(10)**2), unweighted=UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(10)))))
        h.checkvalid()

    def test_validity_StatisticFilter(self):
        h = Histogram([Axis(RegularBinning(10, RealInterval(-5, 5)), statistics=[Statistics(moments=[Moments(InterpretedInlineBuffer.fromarray(numpy.array([0.0])), 1, filter=StatisticFilter(excludes_nan=False))])])], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(10))))
        h.checkvalid()

    def test_validity_Moments(self):
        h = Histogram([Axis(RegularBinning(10, RealInterval(-5, 5)), statistics=[Statistics(moments=[Moments(InterpretedInlineBuffer.fromarray(numpy.array([0.0])), 1), Moments(InterpretedInlineBuffer.fromarray(numpy.array([0.0])), 2)])])], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(10))))
        h.checkvalid()
        h = Histogram([Axis(RegularBinning(10, RealInterval(-5, 5)), statistics=[Statistics(moments=[Moments(InterpretedInlineBuffer.fromarray(numpy.array([0.0])), 0, weightpower=0), Moments(InterpretedInlineBuffer.fromarray(numpy.array([0.0])), 0, weightpower=1)])])], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(10))))
        h.checkvalid()

    def test_validity_Extremes(self):
        h = Histogram([Axis(RegularBinning(10, RealInterval(-5, 5)), statistics=[Statistics(min=Extremes(InterpretedInlineBuffer.fromarray(numpy.array([0.0]))))])], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(10))))
        h.checkvalid()
        h = Histogram([Axis(RegularBinning(10, RealInterval(-5, 5)), statistics=[Statistics(min=Extremes(InterpretedInlineBuffer.fromarray(numpy.array([0.0]))), max=Extremes(InterpretedInlineBuffer.fromarray(numpy.array([0.0]))))])], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(10))))
        h.checkvalid()

    def test_validity_Quantiles(self):
        h = Histogram([Axis(RegularBinning(10, RealInterval(-5, 5)), statistics=[Statistics(quantiles=[Quantiles(InterpretedInlineBuffer.fromarray(numpy.array([0.0])), 0.25), Quantiles(InterpretedInlineBuffer.fromarray(numpy.array([0.0]))), Quantiles(InterpretedInlineBuffer.fromarray(numpy.array([0.0])), 0.75)])])], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(10))))
        h.checkvalid()
        h = Histogram([Axis(RegularBinning(10, RealInterval(-5, 5)), statistics=[Statistics(quantiles=[Quantiles(InterpretedInlineBuffer.fromarray(numpy.array([0.0])), weightpower=0), Quantiles(InterpretedInlineBuffer.fromarray(numpy.array([0.0])), weightpower=1)])])], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(10))))
        h.checkvalid()

    def test_validity_Modes(self):
        h = Histogram([Axis(RegularBinning(10, RealInterval(-5, 5)), statistics=[Statistics(mode=Modes(InterpretedInlineBuffer.fromarray(numpy.array([0.0]))))])], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(10))))
        h.checkvalid()

    def test_validity_Statistics(self):
        h = Histogram([Axis(RegularBinning(10, RealInterval(-5, 5)), statistics=[Statistics()]), Axis(RegularBinning(10, RealInterval(-5, 5)), statistics=[Statistics()])], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(100))))
        h.checkvalid()
        h = Ntuple([Column("one", Column.int32), Column("two", Column.int16)], [NtupleInstance([Chunk([ColumnChunk([Page(RawInlineBuffer(b"\x05\x00\x00\x00"))], [0, 1]), ColumnChunk([Page(RawInlineBuffer(b"\x03\x00"))], [0, 1])])])], column_statistics=[Statistics(moments=[Moments(InterpretedInlineBuffer.fromarray(numpy.array([0.0])), 1), Moments(InterpretedInlineBuffer.fromarray(numpy.array([0.0])), 2)])])
        h.checkvalid()

    def test_validity_Covariance(self):
        h = Histogram([Axis(RegularBinning(10, RealInterval(-5, 5))), Axis(RegularBinning(10, RealInterval(-5, 5)))], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(100))), axis_covariances=[Covariance(0, 1, InterpretedInlineBuffer.fromarray(numpy.arange(1)))])
        h.checkvalid()
        h = Histogram([Axis(RegularBinning(10, RealInterval(-5, 5))), Axis(RegularBinning(10, RealInterval(-5, 5))), Axis(RegularBinning(10, RealInterval(-5, 5)))], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(1000))), axis_covariances=[Covariance(0, 1, InterpretedInlineBuffer.fromarray(numpy.arange(1))), Covariance(0, 2, InterpretedInlineBuffer.fromarray(numpy.arange(1))), Covariance(1, 2, InterpretedInlineBuffer.fromarray(numpy.arange(1)))])
        h.checkvalid()
        h = Histogram([Axis(RegularBinning(10, RealInterval(-5, 5)))], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(10))), [Profile("", Statistics([Moments(InterpretedInlineBuffer.fromarray(numpy.zeros(10)), 1), Moments(InterpretedInlineBuffer.fromarray(numpy.zeros(10)), 2)])), Profile("", Statistics([Moments(InterpretedInlineBuffer.fromarray(numpy.zeros(10)), 1), Moments(InterpretedInlineBuffer.fromarray(numpy.zeros(10)), 2)]))], profile_covariances=[Covariance(0, 1, InterpretedInlineBuffer.fromarray(numpy.arange(1)))])
        h.checkvalid()
        h = Ntuple([Column("one", Column.int32), Column("two", Column.int16)], [NtupleInstance([Chunk([ColumnChunk([Page(RawInlineBuffer(b"\x05\x00\x00\x00"))], [0, 1]), ColumnChunk([Page(RawInlineBuffer(b"\x03\x00"))], [0, 1])])])], column_covariances=[Covariance(0, 1, InterpretedInlineBuffer.fromarray(numpy.arange(1)))])
        h.checkvalid()
        h = Ntuple([Column("one", Column.int32), Column("two", Column.int16)], [NtupleInstance([Chunk([ColumnChunk([Page(RawInlineBuffer(b"\x05\x00\x00\x00"))], [0, 1]), ColumnChunk([Page(RawInlineBuffer(b"\x03\x00"))], [0, 1])])])], column_covariances=[Covariance(0, 1, InterpretedInlineBuffer.fromarray(numpy.arange(1)), weightpower=1), Covariance(0, 1, InterpretedInlineBuffer.fromarray(numpy.arange(1)), weightpower=0)])
        h.checkvalid()

    def test_validity_Profile(self):
        h = Histogram([Axis(RegularBinning(10, RealInterval(-5, 5)))], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(10))), [Profile("", Statistics([Moments(InterpretedInlineBuffer.fromarray(numpy.zeros(10)), 1), Moments(InterpretedInlineBuffer.fromarray(numpy.zeros(10)), 2)]))])
        h.checkvalid()
        h = Histogram([Axis(RegularBinning(10, RealInterval(-5, 5))), Axis(RegularBinning(10, RealInterval(-5, 5)))], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(100))), [Profile("", Statistics([Moments(InterpretedInlineBuffer.fromarray(numpy.zeros(100)), 1), Moments(InterpretedInlineBuffer.fromarray(numpy.zeros(100)), 2)]))])
        h.checkvalid()

    def test_validity_Histogram(self):
        h = Histogram([Axis(RegularBinning(10, RealInterval(-5, 5))), Axis(RegularBinning(10, RealInterval(-5, 5)))], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(100))))
        h.checkvalid()

    def test_validity_Parameter(self):
        h = ParameterizedFunction("x**2", [Parameter("x", InterpretedInlineBuffer.fromarray(numpy.array([5]))), Parameter("y", InterpretedInlineBuffer.fromarray(numpy.array([6])))])
        h.checkvalid()
        assert h.parameters[1].values.array.tolist() == [6]

    def test_validity_ParameterizedFunction(self):
        h = ParameterizedFunction("x**2")
        h.checkvalid()
        h = Histogram([Axis(RegularBinning(10, RealInterval(-5, 5))), Axis(RegularBinning(10, RealInterval(-5, 5)))], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(100))), functions={"f": ParameterizedFunction("x**2", [Parameter("x", InterpretedInlineBuffer.fromarray(numpy.arange(100)))])})
        h.checkvalid()

    def test_validity_EvaluatedFunction(self):
        h = Histogram([Axis(RegularBinning(10, RealInterval(-5, 5))), Axis(RegularBinning(10, RealInterval(-5, 5)))], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(100))), functions={"f": EvaluatedFunction(InterpretedInlineBuffer.fromarray(numpy.arange(100)))})
        h.checkvalid()
        assert h.functions["f"].values.array.tolist() == numpy.arange(100).reshape((10, 10)).tolist()
        h = Histogram([Axis(RegularBinning(10, RealInterval(-5, 5))), Axis(RegularBinning(10, RealInterval(-5, 5)))], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(100))), functions={"f": EvaluatedFunction(InterpretedInlineBuffer.fromarray(numpy.arange(100)), InterpretedInlineBuffer.fromarray(numpy.arange(100)))})
        h.checkvalid()
        h = Histogram([Axis(RegularBinning(10, RealInterval(-5, 5))), Axis(RegularBinning(10, RealInterval(-5, 5)))], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(100))), functions={"f": EvaluatedFunction(InterpretedInlineBuffer.fromarray(numpy.arange(100)), InterpretedInlineBuffer.fromarray(numpy.arange(100)), [Quantiles(InterpretedInlineBuffer.fromarray(numpy.zeros(100)), 0.25), Quantiles(InterpretedInlineBuffer.fromarray(numpy.zeros(100))), Quantiles(InterpretedInlineBuffer.fromarray(numpy.zeros(100)), 0.75)])})
        h.checkvalid()

    def test_validity_BinnedEvaluatedFunction(self):
        h = BinnedEvaluatedFunction([Axis()], InterpretedInlineBuffer(numpy.array([0.0]), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [0.0]
        h = BinnedEvaluatedFunction([Axis(), Axis()], InterpretedInlineBuffer(numpy.array([[0.0]]), dtype=InterpretedInlineBuffer.float64))
        h.checkvalid()
        assert h.values.array.tolist() == [[0.0]]

    def test_validity_Page(self):
        h = Ntuple([Column("one", Column.int32)], [NtupleInstance([Chunk([ColumnChunk([Page(RawInlineBuffer(b"\x05\x00\x00\x00"))], [0, 1])])])])
        h.checkvalid()
        assert h.instances[0].chunks[0].column_chunks[0].pages[0].array.tolist() == [5]

        h = Ntuple([Column("one", Column.int32)], [NtupleInstance([Chunk([ColumnChunk([], [0])])])])
        h.checkvalid()
        assert h.instances[0].chunks[0].column_chunks[0].array.tolist() == []
        assert {n: x.tolist() for n, x in h.instances[0].chunks[0].arrays.items()} == {"one": []}
        for arrays in h.instances[0].arrays: pass
        assert {n: x.tolist() for n, x in arrays.items()} == {"one": []}

        h = Ntuple([Column("one", Column.int32)], [NtupleInstance([Chunk([ColumnChunk([Page(RawInlineBuffer(b"\x05\x00\x00\x00"))], [0, 1])])])])
        h.checkvalid()
        assert h.instances[0].chunks[0].column_chunks[0].array.tolist() == [5]
        assert {n: x.tolist() for n, x in h.instances[0].chunks[0].arrays.items()} == {"one": [5]}
        for arrays in h.instances[0].arrays: pass
        assert {n: x.tolist() for n, x in arrays.items()} == {"one": [5]}

        h = Ntuple([Column("one", Column.int32)], [NtupleInstance([Chunk([ColumnChunk([Page(RawInlineBuffer(b"\x05\x00\x00\x00")), Page(RawInlineBuffer(b"\x04\x00\x00\x00\x03\x00\x00\x00"))], [0, 1, 3])])])])
        h.checkvalid()
        assert h.instances[0].chunks[0].column_chunks[0].array.tolist() == [5, 4, 3]
        assert {n: x.tolist() for n, x in h.instances[0].chunks[0].arrays.items()} == {"one": [5, 4, 3]}
        for arrays in h.instances[0].arrays: pass
        assert {n: x.tolist() for n, x in arrays.items()} == {"one": [5, 4, 3]}

    def test_validity_Chunk(self):
        h = Ntuple([Column("one", Column.float64)], [NtupleInstance([Chunk([ColumnChunk([], [0])])])])
        h.checkvalid()

        h = Ntuple([Column("one", Column.int32)], [NtupleInstance([])])
        h.checkvalid()
        for arrays in h.instances[0].arrays:
            assert False

        h = Ntuple([Column("one", Column.int32)], [NtupleInstance([Chunk([ColumnChunk([Page(RawInlineBuffer(b"\x05\x00\x00\x00"))], [0, 1])])])])
        h.checkvalid()
        for arrays in h.instances[0].arrays: pass
        assert {n: x.tolist() for n, x in arrays.items()} == {"one": [5]}

        h = Ntuple([Column("one", Column.int32)], [NtupleInstance([Chunk([ColumnChunk([Page(RawInlineBuffer(b"\x05\x00\x00\x00"))], [0, 1])]), Chunk([ColumnChunk([Page(RawInlineBuffer(b"\x05\x00\x00\x00"))], [0, 1])])])])
        h.checkvalid()
        for arrays in h.instances[0].arrays:
            assert {n: x.tolist() for n, x in arrays.items()} == {"one": [5]}

    def test_validity_Column(self):
        h = Ntuple([Column("one", Column.float64), Column("two", Column.int32)], [NtupleInstance([])])
        h.checkvalid()

        h = Ntuple([Column("one", Column.int32), Column("two", Column.int16)], [NtupleInstance([Chunk([ColumnChunk([Page(RawInlineBuffer(b"\x05\x00\x00\x00"))], [0, 1]), ColumnChunk([Page(RawInlineBuffer(b"\x03\x00"))], [0, 1])])])])
        h.checkvalid()
        for arrays in h.instances[0].arrays: pass
        assert {n: x.tolist() for n, x in arrays.items()} == {"one": [5], "two": [3]}

    def test_validity_Ntuple(self):
        h = Ntuple([Column("one", Column.float64)], [NtupleInstance([])])
        h.checkvalid()

    def test_validity_collection(self):
        h = Collection()
        h.checkvalid()
        h = Collection({"id": Histogram([Axis(RegularBinning(10, RealInterval(-5, 5)))], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(30)))), "id2": Histogram([Axis(RegularBinning(100, RealInterval(-5, 5)))], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(300))))}, axis=[Axis(RegularBinning(3, RealInterval(-1, 1)))])
        h.checkvalid()
        h = Collection({"b": Collection({"c": Histogram([Axis(RegularBinning(10, RealInterval(-5, 5)))], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(60)))), "d": Histogram([Axis(RegularBinning(100, RealInterval(-5, 5)))], UnweightedCounts(InterpretedInlineBuffer.fromarray(numpy.arange(600))))}, axis=[Axis(FractionBinning())])}, axis=[Axis(RegularBinning(3, RealInterval(-1, 1)))])
        h.checkvalid()
        assert h.objects["b"].objects["c"].counts.counts.array.tolist() == numpy.arange(60).reshape((3, 2, 10)).tolist()



#!/usr/bin/env python

# BSD 3-Clause License; see https://github.com/scikit-hep/aghast/blob/master/LICENSE

import pytest
import numpy

import aghast
from aghast import *

ROOT = pytest.importorskip("ROOT")
connect = pytest.importorskip("aghast._connect._root")

data = [-0.319, -1.949, -1.511, 1.176, 0.695, -0.286, 0.392, -0.094, -0.714, 1.223, 1.811, -2.57, -0.014, 0.328, -0.084, -0.13, -0.751, 0.47, 1.558, 0.714, -0.999, 0.336, -2.51, -0.532, 2.495, -0.085, 0.165, 0.764, 0.865, -0.604, -1.083, 2.501, -1.074, -0.853, -0.241, -0.346, 0.472, -0.017, 1.013, 0.887, 2.154, -1.354, 0.966, -1.118, -1.374, -0.928, -0.664, 2.803, -0.133, 1.814, 0.844, -0.719, 1.239, 0.49, 0.333, 0.518, 0.079, -1.33, -0.048, 1.335, -2.108, -1.772, -0.68, 0.151, -0.479, 0.749, 0.589, -1.048, -0.491, 1.125, 1.064, 0.4, 0.349, 0.193, -0.645, 0.038, 0.536, -0.675, 0.732, -1.442, -0.889, -0.976, 0.889, 1.296, 1.231, 0.934, -1.359, 1.602, 0.186, -0.622, -0.08, -0.887, 0.109, 0.418, 0.945, -0.081, -0.32, -1.309, -0.497, 0.346]

def check1d(before, after):
    assert before.GetNbinsX() == after.GetNbinsX()
    for i in range(before.GetNbinsX() + 2):
        assert before.GetBinContent(i) == after.GetBinContent(i)
        assert before.GetBinError(i) == after.GetBinError(i)
    assert before.GetEntries() == after.GetEntries()
    assert before.GetMean() == after.GetMean()
    assert before.GetStdDev() == after.GetStdDev()
    assert before.GetTitle() == after.GetTitle()
    assert before.GetXaxis().GetTitle() == after.GetXaxis().GetTitle()
    assert bool(before.GetXaxis().GetLabels()) == bool(after.GetXaxis().GetLabels())
    assert before.GetXaxis().IsVariableBinSize() == after.GetXaxis().IsVariableBinSize()
    if before.GetXaxis().GetLabels():
        assert list(before.GetXaxis().GetLabels()) == list(after.GetXaxis().GetLabels())
    elif before.GetXaxis().IsVariableBinSize():
        beforeedges = numpy.full(before.GetNbinsX() + 1, 999, dtype=numpy.float64)
        before.GetXaxis().GetLowEdge(beforeedges)
        beforeedges[-1] = before.GetXaxis().GetBinUpEdge(before.GetNbinsX())
        afteredges = numpy.full(after.GetNbinsX() + 1, 123, dtype=numpy.float64)
        after.GetXaxis().GetLowEdge(afteredges)
        afteredges[-1] = after.GetXaxis().GetBinUpEdge(after.GetNbinsX())
        assert numpy.array_equal(before, after)
    else:
        assert before.GetXaxis().GetBinLowEdge(1) == after.GetXaxis().GetBinLowEdge(1)
        assert before.GetXaxis().GetBinUpEdge(before.GetNbinsX()) == after.GetXaxis().GetBinUpEdge(after.GetNbinsX())

num = 0
def rootname():
    global num
    num += 1
    return "name-{0}".format(num)

@pytest.mark.parametrize("cls", [ROOT.TH1C, ROOT.TH1S, ROOT.TH1I, ROOT.TH1F, ROOT.TH1D])
def test_root_oned(cls):
    before = cls(rootname(), "title", 5, -2.0, 2.0)
    before.GetXaxis().SetTitle("title2")
    after = aghast.to_root(aghast.from_root(before), rootname())
    check1d(before, after)

    before = cls(rootname(), "title", 5, -2.0, 2.0)
    before.GetXaxis().SetTitle("title2")
    for x in data: before.Fill(x)
    after = aghast.to_root(aghast.from_root(before), rootname())
    check1d(before, after)

    before = cls(rootname(), "title", 5, -2.0, 2.0)
    before.GetXaxis().SetTitle("title2")
    for i, x in enumerate(["one", "two", "three", "four", "five"]):
        before.GetXaxis().SetBinLabel(i + 1, x)
    after = aghast.to_root(aghast.from_root(before), rootname())
    check1d(before, after)

    before = cls(rootname(), "title", 5, -5.0, 5.0)
    before.GetXaxis().SetTitle("title2")
    for i, x in enumerate(["one", "two", "three", "four", "five"]):
        before.GetXaxis().SetBinLabel(i + 1, x)
    for x in data: before.Fill(x)
    after = aghast.to_root(aghast.from_root(before), rootname())
    check1d(before, after)

    edges = numpy.array([-5.0, -3.0, 0.0, 5.0, 10.0, 100.0], dtype=numpy.float64)
    before = cls(rootname(), "title", 5, edges)
    before.GetXaxis().SetTitle("title2")
    after = aghast.to_root(aghast.from_root(before), rootname())
    check1d(before, after)

    edges = numpy.array([-5.0, -3.0, 0.0, 5.0, 10.0, 100.0], dtype=numpy.float64)
    before = cls(rootname(), "title", 5, edges)
    before.GetXaxis().SetTitle("title2")
    for x in data: before.Fill(x)
    after = aghast.to_root(aghast.from_root(before), rootname())
    check1d(before, after)



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

import flatbuffers

class IrregularBinning(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsIrregularBinning(cls, buf, offset):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = IrregularBinning()
        x.Init(buf, n + offset)
        return x

    # IrregularBinning
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # IrregularBinning
    def Intervals(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 24
            from .RealInterval import RealInterval
            obj = RealInterval()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # IrregularBinning
    def IntervalsLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # IrregularBinning
    def Overflow(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            x = o + self._tab.Pos
            from .RealOverflow import RealOverflow
            obj = RealOverflow()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # IrregularBinning
    def OverlappingFill(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int8Flags, o + self._tab.Pos)
        return 0

def IrregularBinningStart(builder): builder.StartObject(3)
def IrregularBinningAddIntervals(builder, intervals): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(intervals), 0)
def IrregularBinningStartIntervalsVector(builder, numElems): return builder.StartVector(24, numElems, 8)
def IrregularBinningAddOverflow(builder, overflow): builder.PrependStructSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(overflow), 0)
def IrregularBinningAddOverlappingFill(builder, overlappingFill): builder.PrependInt8Slot(2, overlappingFill, 0)
def IrregularBinningEnd(builder): return builder.EndObject()



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

import flatbuffers

class Chunk(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsChunk(cls, buf, offset):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = Chunk()
        x.Init(buf, n + offset)
        return x

    # Chunk
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Chunk
    def ColumnChunks(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .ColumnChunk import ColumnChunk
            obj = ColumnChunk()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Chunk
    def ColumnChunksLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # Chunk
    def Metadata(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Metadata import Metadata
            obj = Metadata()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

def ChunkStart(builder): builder.StartObject(2)
def ChunkAddColumnChunks(builder, columnChunks): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(columnChunks), 0)
def ChunkStartColumnChunksVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def ChunkAddMetadata(builder, metadata): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(metadata), 0)
def ChunkEnd(builder): return builder.EndObject()



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

import flatbuffers

class FractionBinning(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsFractionBinning(cls, buf, offset):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = FractionBinning()
        x.Init(buf, n + offset)
        return x

    # FractionBinning
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # FractionBinning
    def Layout(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int8Flags, o + self._tab.Pos)
        return 0

    # FractionBinning
    def LayoutReversed(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return bool(self._tab.Get(flatbuffers.number_types.BoolFlags, o + self._tab.Pos))
        return False

    # FractionBinning
    def ErrorMethod(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int8Flags, o + self._tab.Pos)
        return 0

def FractionBinningStart(builder): builder.StartObject(3)
def FractionBinningAddLayout(builder, layout): builder.PrependInt8Slot(0, layout, 0)
def FractionBinningAddLayoutReversed(builder, layoutReversed): builder.PrependBoolSlot(1, layoutReversed, 0)
def FractionBinningAddErrorMethod(builder, errorMethod): builder.PrependInt8Slot(2, errorMethod, 0)
def FractionBinningEnd(builder): return builder.EndObject()



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

import flatbuffers

class RealInterval(object):
    __slots__ = ['_tab']

    # RealInterval
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # RealInterval
    def Low(self): return self._tab.Get(flatbuffers.number_types.Float64Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(0))
    # RealInterval
    def High(self): return self._tab.Get(flatbuffers.number_types.Float64Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(8))
    # RealInterval
    def LowInclusive(self): return self._tab.Get(flatbuffers.number_types.BoolFlags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(16))
    # RealInterval
    def HighInclusive(self): return self._tab.Get(flatbuffers.number_types.BoolFlags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(17))

def CreateRealInterval(builder, low, high, lowInclusive, highInclusive):
    builder.Prep(8, 24)
    builder.Pad(6)
    builder.PrependBool(highInclusive)
    builder.PrependBool(lowInclusive)
    builder.PrependFloat64(high)
    builder.PrependFloat64(low)
    return builder.Offset()



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

import flatbuffers

class RegularBinning(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsRegularBinning(cls, buf, offset):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = RegularBinning()
        x.Init(buf, n + offset)
        return x

    # RegularBinning
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # RegularBinning
    def Num(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint64Flags, o + self._tab.Pos)
        return 0

    # RegularBinning
    def Interval(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            x = o + self._tab.Pos
            from .RealInterval import RealInterval
            obj = RealInterval()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # RegularBinning
    def Overflow(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            x = o + self._tab.Pos
            from .RealOverflow import RealOverflow
            obj = RealOverflow()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # RegularBinning
    def Circular(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return bool(self._tab.Get(flatbuffers.number_types.BoolFlags, o + self._tab.Pos))
        return False

def RegularBinningStart(builder): builder.StartObject(4)
def RegularBinningAddNum(builder, num): builder.PrependUint64Slot(0, num, 0)
def RegularBinningAddInterval(builder, interval): builder.PrependStructSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(interval), 0)
def RegularBinningAddOverflow(builder, overflow): builder.PrependStructSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(overflow), 0)
def RegularBinningAddCircular(builder, circular): builder.PrependBoolSlot(3, circular, 0)
def RegularBinningEnd(builder): return builder.EndObject()



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

import flatbuffers

# ///////////////////////////////////////////////// axis
class Axis(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsAxis(cls, buf, offset):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = Axis()
        x.Init(buf, n + offset)
        return x

    # Axis
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Axis
    def BinningType(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, o + self._tab.Pos)
        return 0

    # Axis
    def Binning(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            from flatbuffers.table import Table
            obj = Table(bytearray(), 0)
            self._tab.Union(obj, o)
            return obj
        return None

    # Axis
    def Expression(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Axis
    def Statistics(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .Statistics import Statistics
            obj = Statistics()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Axis
    def StatisticsLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # Axis
    def Title(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Axis
    def Metadata(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Metadata import Metadata
            obj = Metadata()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Axis
    def Decoration(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from .Decoration import Decoration
            obj = Decoration()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

def AxisStart(builder): builder.StartObject(7)
def AxisAddBinningType(builder, binningType): builder.PrependUint8Slot(0, binningType, 0)
def AxisAddBinning(builder, binning): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(binning), 0)
def AxisAddExpression(builder, expression): builder.PrependUOffsetTRelativeSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(expression), 0)
def AxisAddStatistics(builder, statistics): builder.PrependUOffsetTRelativeSlot(3, flatbuffers.number_types.UOffsetTFlags.py_type(statistics), 0)
def AxisStartStatisticsVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def AxisAddTitle(builder, title): builder.PrependUOffsetTRelativeSlot(4, flatbuffers.number_types.UOffsetTFlags.py_type(title), 0)
def AxisAddMetadata(builder, metadata): builder.PrependUOffsetTRelativeSlot(5, flatbuffers.number_types.UOffsetTFlags.py_type(metadata), 0)
def AxisAddDecoration(builder, decoration): builder.PrependUOffsetTRelativeSlot(6, flatbuffers.number_types.UOffsetTFlags.py_type(decoration), 0)
def AxisEnd(builder): return builder.EndObject()



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

import flatbuffers

# ///////////////////////////////////////////////// histogram
class Histogram(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsHistogram(cls, buf, offset):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = Histogram()
        x.Init(buf, n + offset)
        return x

    # Histogram
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Histogram
    def Axis(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .Axis import Axis
            obj = Axis()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Histogram
    def AxisLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # Histogram
    def CountsType(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, o + self._tab.Pos)
        return 0

    # Histogram
    def Counts(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            from flatbuffers.table import Table
            obj = Table(bytearray(), 0)
            self._tab.Union(obj, o)
            return obj
        return None

    # Histogram
    def Profile(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .Profile import Profile
            obj = Profile()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Histogram
    def ProfileLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # Histogram
    def AxisCovariances(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .Covariance import Covariance
            obj = Covariance()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Histogram
    def AxisCovariancesLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # Histogram
    def ProfileCovariances(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .Covariance import Covariance
            obj = Covariance()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Histogram
    def ProfileCovariancesLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # Histogram
    def FunctionsLookup(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.String(a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 4))
        return ""

    # Histogram
    def FunctionsLookupLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # Histogram
    def Functions(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(18))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .Function import Function
            obj = Function()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Histogram
    def FunctionsLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(18))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

def HistogramStart(builder): builder.StartObject(8)
def HistogramAddAxis(builder, axis): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(axis), 0)
def HistogramStartAxisVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def HistogramAddCountsType(builder, countsType): builder.PrependUint8Slot(1, countsType, 0)
def HistogramAddCounts(builder, counts): builder.PrependUOffsetTRelativeSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(counts), 0)
def HistogramAddProfile(builder, profile): builder.PrependUOffsetTRelativeSlot(3, flatbuffers.number_types.UOffsetTFlags.py_type(profile), 0)
def HistogramStartProfileVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def HistogramAddAxisCovariances(builder, axisCovariances): builder.PrependUOffsetTRelativeSlot(4, flatbuffers.number_types.UOffsetTFlags.py_type(axisCovariances), 0)
def HistogramStartAxisCovariancesVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def HistogramAddProfileCovariances(builder, profileCovariances): builder.PrependUOffsetTRelativeSlot(5, flatbuffers.number_types.UOffsetTFlags.py_type(profileCovariances), 0)
def HistogramStartProfileCovariancesVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def HistogramAddFunctionsLookup(builder, functionsLookup): builder.PrependUOffsetTRelativeSlot(6, flatbuffers.number_types.UOffsetTFlags.py_type(functionsLookup), 0)
def HistogramStartFunctionsLookupVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def HistogramAddFunctions(builder, functions): builder.PrependUOffsetTRelativeSlot(7, flatbuffers.number_types.UOffsetTFlags.py_type(functions), 0)
def HistogramStartFunctionsVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def HistogramEnd(builder): return builder.EndObject()



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

import flatbuffers

class VariationBinning(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsVariationBinning(cls, buf, offset):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = VariationBinning()
        x.Init(buf, n + offset)
        return x

    # VariationBinning
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # VariationBinning
    def Variations(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from .Variation import Variation
            obj = Variation()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # VariationBinning
    def VariationsLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # VariationBinning
    def SystematicUnits(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int8Flags, o + self._tab.Pos)
        return 0

    # VariationBinning
    def SystematicNames(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.String(a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 4))
        return ""

    # VariationBinning
    def SystematicNamesLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # VariationBinning
    def CategorySystematicNames(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.String(a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 4))
        return ""

    # VariationBinning
    def CategorySystematicNamesLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

def VariationBinningStart(builder): builder.StartObject(4)
def VariationBinningAddVariations(builder, variations): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(variations), 0)
def VariationBinningStartVariationsVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def VariationBinningAddSystematicUnits(builder, systematicUnits): builder.PrependInt8Slot(1, systematicUnits, 0)
def VariationBinningAddSystematicNames(builder, systematicNames): builder.PrependUOffsetTRelativeSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(systematicNames), 0)
def VariationBinningStartSystematicNamesVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def VariationBinningAddCategorySystematicNames(builder, categorySystematicNames): builder.PrependUOffsetTRelativeSlot(3, flatbuffers.number_types.UOffsetTFlags.py_type(categorySystematicNames), 0)
def VariationBinningStartCategorySystematicNamesVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def VariationBinningEnd(builder): return builder.EndObject()



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: aghast_generated

import flatbuffers

class Covariance(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsCovariance(cls, buf, offset):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = Covariance()
        x.Init(buf, n + offset)
        return x

    # Covariance
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Covariance
    def Xindex(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int32Flags, o + self._tab.Pos)
        return 0

    # Covariance
    def Yindex(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int32Flags, o + self._tab.Pos)
        return 0

    # Covariance
    def SumwxyType(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, o + self._tab.Pos)
        return 0

    # Covariance
    def Sumwxy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            from flatbuffers.table import Table
            obj = Table(bytearray(), 0)
            self._tab.Union(obj, o)
            return obj
        return None

    # Covariance
    def Weightpower(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int8Flags, o + self._tab.Pos)
        return 0

    # Covariance
    def Filter(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            x = o + self._tab.Pos
            from .StatisticFilter import StatisticFilter
            obj = StatisticFilter()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

def CovarianceStart(builder): builder.StartObject(6)
def CovarianceAddXindex(builder, xindex): builder.PrependInt32Slot(0, xindex, 0)
def CovarianceAddYindex(builder, yindex): builder.PrependInt32Slot(1, yindex, 0)
def CovarianceAddSumwxyType(builder, sumwxyType): builder.PrependUint8Slot(2, sumwxyType, 0)
def CovarianceAddSumwxy(builder, sumwxy): builder.PrependUOffsetTRelativeSlot(3, flatbuffers.number_types.UOffsetTFlags.py_type(sumwxy), 0)
def CovarianceAddWeightpower(builder, weightpower): builder.PrependInt8Slot(4, weightpower, 0)
def CovarianceAddFilter(builder, filter): builder.PrependStructSlot(5, flatbuffers.number_types.UOffsetTFlags.py_type(filter), 0)
def CovarianceEnd(builder): return builder.EndObject()
